@@ -73,12 +73,22 @@ rmDerivedMspct <- function(x) {
 #' @family set and unset 'multi spectral' class functions
 #'
 shared_member_class <- function(l, target.set = spct_classes()) {
-  l.class <- target.set
-  for (i in seq_along(l)) {
-    member_class <- class(l[[i]])
-    l.class <- intersect(l.class, member_class)
+  if (length(l) == 0) {
+    if (is.generic_mspct(l)) {
+      # we return the least derived member class allowed
+      gsub("_mspct$", "_spct", class(l)[1])
+    } else {
+      character()
+    }
+  } else {
+    # we inspect the classes of members
+    l.class <- target.set
+    for (i in seq_along(l)) {
+      member_class <- class(l[[i]])
+      l.class <- intersect(l.class, member_class)
+    }
+    l.class
   }
-  l.class
 }
 
 # Constructors ------------------------------------------------------------
@@ -107,8 +117,10 @@ shared_member_class <- function(l, target.set = spct_classes()) {
 #' @examples
 #' filter_mspct(list(polyester.spct, yellow_gel.spct))
 #'
-generic_mspct <- function(l = NULL, class = "generic_spct",
-                          ncol = 1, byrow = FALSE,
+generic_mspct <- function(l = NULL,
+                          class = "generic_spct",
+                          ncol = 1,
+                          byrow = FALSE,
                           dim = c(length(l) %/% ncol, ncol)) {
   if (is.generic_spct(l)) {
     l <- list(l)
@@ -157,7 +169,10 @@ generic_mspct <- function(l = NULL, class = "generic_spct",
 #' @export
 #'
 #'
-calibration_mspct <- function(l = NULL, ncol = 1, byrow = FALSE, ...) {
+calibration_mspct <- function(l = NULL,
+                              ncol = 1,
+                              byrow = FALSE,
+                              ...) {
   generic_mspct(l, class = "calibration_spct", ncol = ncol, byrow = byrow)
 }
 
@@ -166,7 +181,10 @@ calibration_mspct <- function(l = NULL, ncol = 1, byrow = FALSE, ...) {
 #' @export
 #'
 #'
-raw_mspct <- function(l = NULL, ncol = 1, byrow = FALSE, ...) {
+raw_mspct <- function(l = NULL,
+                      ncol = 1,
+                      byrow = FALSE,
+                      ...) {
   generic_mspct(l, class = "raw_spct", ncol = ncol, byrow = byrow)
 }
 
@@ -175,7 +193,9 @@ raw_mspct <- function(l = NULL, ncol = 1, byrow = FALSE, ...) {
 #' @export
 #'
 #'
-cps_mspct <- function(l = NULL, ncol = 1, byrow = FALSE, ...) {
+cps_mspct <- function(l = NULL,
+                      ncol = 1,
+                      byrow = FALSE, ...) {
   generic_mspct(l, class = "cps_spct", ncol = ncol, byrow = byrow)
 }
 
@@ -184,7 +204,10 @@ cps_mspct <- function(l = NULL, ncol = 1, byrow = FALSE, ...) {
 #' @export
 #'
 #'
-source_mspct <- function(l = NULL, ncol = 1, byrow = FALSE, ...) {
+source_mspct <- function(l = NULL,
+                         ncol = 1,
+                         byrow = FALSE,
+                         ...) {
   generic_mspct(l, class = "source_spct", ncol = ncol, byrow = byrow)
 }
 
@@ -193,7 +216,10 @@ source_mspct <- function(l = NULL, ncol = 1, byrow = FALSE, ...) {
 #' @export
 #'
 #'
-filter_mspct <- function(l = NULL, ncol = 1, byrow = FALSE, ...) {
+filter_mspct <- function(l = NULL,
+                         ncol = 1,
+                         byrow = FALSE,
+                         ...) {
   generic_mspct(l, class = "filter_spct", ncol = ncol, byrow = byrow)
 }
 
@@ -202,7 +228,10 @@ filter_mspct <- function(l = NULL, ncol = 1, byrow = FALSE, ...) {
 #' @export
 #'
 #'
-reflector_mspct <- function(l = NULL, ncol = 1, byrow = FALSE, ...) {
+reflector_mspct <- function(l = NULL,
+                            ncol = 1,
+                            byrow = FALSE,
+                            ...) {
   generic_mspct(l, class = "reflector_spct", ncol = ncol, byrow = byrow)
 }
 
@@ -211,7 +240,10 @@ reflector_mspct <- function(l = NULL, ncol = 1, byrow = FALSE, ...) {
 #' @export
 #'
 #'
-object_mspct <- function(l = NULL, ncol = 1, byrow = FALSE, ...) {
+object_mspct <- function(l = NULL,
+                         ncol = 1,
+                         byrow = FALSE,
+                         ...) {
   generic_mspct(l, class = "object_spct", ncol = ncol, byrow = byrow)
 }
 
@@ -220,7 +252,10 @@ object_mspct <- function(l = NULL, ncol = 1, byrow = FALSE, ...) {
 #' @export
 #'
 #'
-response_mspct <- function(l = NULL, ncol = 1, byrow = FALSE, ...) {
+response_mspct <- function(l = NULL,
+                           ncol = 1,
+                           byrow = FALSE,
+                           ...) {
   generic_mspct(l, class = "response_spct", ncol = ncol, byrow = byrow)
 }
 
@@ -229,7 +264,10 @@ response_mspct <- function(l = NULL, ncol = 1, byrow = FALSE, ...) {
 #' @export
 #'
 #'
-chroma_mspct <- function(l = NULL, ncol = 1, byrow = FALSE, ...) {
+chroma_mspct <- function(l = NULL,
+                         ncol = 1,
+                         byrow = FALSE,
+                         ...) {
   generic_mspct(l, class = "chroma_spct", ncol = ncol, byrow = byrow)
 }
 
